﻿using LpCore;
using LpSdk;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DemoSDK
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            //获取脚本目录中的脚本文件
            List<string> lp2files = LpSdk.RunTask.GetScripList();
            foreach (string s in lp2files)
            {
                this.cbLp2.Items.Add(System.IO.Path.GetFileNameWithoutExtension(s));
            }

            ///日志的注册事件
            LpSdk.RunTask.SetTaskProcessAction = SetTaskProcessAction;

            ///脚本任务完成后通知
            LpSdk.RunTask.SetTaskEndAction = SetTaskEndAction;
        }

        /// <summary>
        /// 任务运行配置，包含运行器位置，日志方式，脚本信息，变量信息
        /// </summary>
        LpSdk.TaskConfig taskConfig = new TaskConfig();

        private void btnTest_Click(object sender, EventArgs e)
        {
            try
            {
                this.SetConfig();
                this.btnAsyTest.Enabled = this.btnTest.Enabled = false;
                this.rtbResult.Clear();
                this.SetLog("开始同步运行脚本\r\n", Color.Green);
                LpSdk.RunTask.StartTask(this.taskConfig);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "出错");
            }
            finally
            {
                this.btnAsyTest.Enabled = this.btnTest.Enabled = true;
            }
        }

        /// <summary>
        /// 异步运行脚本
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAsyTest_Click(object sender, EventArgs e)
        {
            try
            {
                this.SetConfig();
                this.rtbResult.Clear();
                int pid = LpSdk.RunTask.AsyncStartTask(this.taskConfig);
                this.SetLog("当前开始异步运行，脚本运行器进程id为" + pid.ToString() + "\r\n", Color.Green);
                this.btnAsyTest.Enabled = this.btnTest.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "出错");
            }
        }

        private void SetConfig()
        {
            if (this.cbLp2.SelectedIndex == -1) throw new Exception("请先选择一个脚本");
            //脚本文件完整路径
            this.taskConfig.TaskFile = LpCore.Common.ProjectDir + this.cbLp2.Text + ".lp2";
            this.taskConfig.IsShowWindow = true;
            this.taskConfig.TaskName = "测试任务名";
            this.taskConfig.IsMute = true;
            this.taskConfig.SKey = this.tbSKey.Text;
        }

        /// <summary>
        /// 任务结束时返回的信息
        /// </summary>
        /// <param name="pid">进程id</param>
        /// <param name="endType">结束类型</param>
        /// <param name="list">变量列表</param>
        /// <param name="s">结束日志</param>
        void SetTaskEndAction(int pid, LpSdk.Enum.EndType endType, List<LpSdk.VariableInfo> list, string s)
        {
            string end = s == "" ? "任务运行结束" : string.Format("任务运行结束：{0}\r\n", s);
            this.SetLog(end, Color.Blue);
            ///如果设置了保存日志文件，可以在这里打开
            if (System.IO.File.Exists(this.taskConfig.LogFilePath)) System.Diagnostics.Process.Start(this.taskConfig.LogFilePath);
            if (!this.IsDisposed)
            {
                try
                {
                    //日志都是异步调用的，界面操作需要回主线程
                    this.Invoke((EventHandler)delegate
                    {
                        this.btnAsyTest.Enabled = this.btnTest.Enabled = true;
                    });
                }
                catch { }
            }
        }

        /// <summary>
        /// 显示脚本运行步骤
        /// </summary>
        /// <param name="pid">进程id</param>
        /// <param name="stepid">步骤id</param>
        /// <param name="stepname">步骤名</param>
        /// <param name="actionType">操作类型</param>
        /// <param name="text">日志类型</param>
        /// <param name="logType">成功true失败false</param>
        void SetTaskProcessAction(int pid, int stepid, string stepname, LpSdk.Enum.ActionType actionType, string text, bool logType)
        {
            Color c = Color.Green;
            if (!logType) c = Color.Red;
            this.SetLog("[PID:" + pid + "]—" + text + "\r\n", c);
        }

        /// <summary>
        /// 显示运行日志
        /// </summary>
        /// <param name="s"></param>
        /// <param name="c"></param>
        private void SetLog(string s, Color c)
        {
            if (rtbResult.InvokeRequired)
            {
                Invoke(new Action<string, Color>(SetLog), new object[] { s, c });
            }
            else
            {
                rtbResult.SelectionStart = rtbResult.TextLength;
                rtbResult.SelectionColor = c;
                rtbResult.AppendText(s);
                rtbResult.ScrollToCaret();
                if (rtbResult.Lines.Length > 500)
                {
                    rtbResult.Clear();
                }
            }
        }

        /// <summary>
        /// 选择不同脚本时，将其中的变量显示在列表中
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbLp2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbLp2.SelectedIndex == -1) return;
            //默认的脚本目录为 LpCore.Common.ProjectDir
            string lp2file = LpCore.Common.ProjectDir + this.cbLp2.Text + ".lp2";
            if (!System.IO.File.Exists(lp2file))
            {
                MessageBox.Show("不存在脚本文件：" + lp2file);
                return;
            }
            List<VariableInfo> varlist = LpSdk.RunTask.GetScriptVarList(lp2file);
            this.taskConfig.VarInfo = varlist;

            List<string> varnames = new List<string>();
            foreach (VariableInfo vi in varlist) varnames.Add(vi.VariableName);
            this.lbVarList.DataSource = varnames;
        }

        /// <summary>
        /// 将选中变量值显示出来
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lbVarList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbVarList.SelectedIndex == -1) return;
            VariableInfo vi = this.taskConfig.VarInfo.Find((a) => a.VariableName == this.lbVarList.Text);
            switch (vi.VariableType)
            {
                case LpSdk.Enum.VariableType.String:
                    this.tbValue.Text = vi.VariableStr;
                    this.lbVarType.Text = "字符";
                    break;
                case LpSdk.Enum.VariableType.Int:
                    this.tbValue.Text = vi.VariableInt.ToString();
                    this.lbVarType.Text = "计数器";
                    break;
                case LpSdk.Enum.VariableType.List:
                    this.tbValue.Text = string.Join("\r\n", vi.VariableList.ToArray());
                    this.lbVarType.Text = "列表";
                    break;
            }
        }

        /// <summary>
        /// 保存变量值
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            if (lbVarList.SelectedIndex == -1)
            {
                MessageBox.Show("请先选择一个变量");
                return;
            }
            VariableInfo vi = this.taskConfig.VarInfo.Find((a) => a.VariableName == this.lbVarList.Text);
            switch (vi.VariableType)
            {
                case LpSdk.Enum.VariableType.String:
                    vi.VariableStr = this.tbValue.Text;
                    break;
                case LpSdk.Enum.VariableType.Int:
                    int i = 0;
                    if(int.TryParse(this.tbValue.Text,out i))
                    {
                        vi.VariableInt = i;
                    }
                    else
                    {
                        MessageBox.Show("该变量为计数器变量，请写数字");
                        return;
                    }
                    break;
                case LpSdk.Enum.VariableType.List:
                    vi.VariableList = this.tbValue.Lines.ToList();//.Text.Replace("")
                    break;
            }
        }
    }
}